package com.nisum.bookstore.controller;

import com.nisum.bookstore.dto.BookDto;
import com.nisum.bookstore.exceptions.BookNotFoundException;
import com.nisum.bookstore.service.BookService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@RestController
@RequestMapping(("/books"))
public class BookController {

    @Autowired
    private BookService bookService;

    @GetMapping
    public Flux<BookDto> getBooks() {
        return bookService.getBooks().log();
    }

    @GetMapping("/dummy/{id}")
    public Mono<BookDto> getBookDummyById( @PathVariable String id ) {
        return bookService.getBookById( id ).log().switchIfEmpty(
                Mono.just( new BookDto("1", "name", "author", "publications", "description") ) );
    }

    @GetMapping("/{id}")
    public Mono<BookDto> getBookById( @PathVariable String id ) {
        return bookService.getBookById( id ).log().switchIfEmpty(
            BookNotFoundException.monoResponseBookNotFoundException( id )
        );
    }

    @PostMapping
    public Mono<BookDto> addBook( @RequestBody BookDto bookDto ) {
        return bookService.addBook( Mono.just(bookDto) ).log();
    }

    @PutMapping( "/{id}" )
    public Mono<BookDto> updateBook( @PathVariable String id, @RequestBody BookDto bookDto ) {
        return bookService.updateBook( id, Mono.just(bookDto) ).log();
    }

    @DeleteMapping( "/{id}" )
    public Mono<Void> deleteBook( @PathVariable String id ) {
        return bookService.deleteBook( id ).log();
    }

}
